/* Caja x2goumount
 * Copyright (C) 2005 The GNOME Foundation
 * Copyright (C) 2007-2015 Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>
 * Copyright (C) 2007-2015 Heinz-Markus Graesing <heinz-m.graesing@obviously-nice.de>
 * Copyright (C) 2013-2015 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 *
 * This Program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this Library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <glib/gi18n.h>
#include <string.h>
#include <caja/libcaja-extension/caja-extension-types.h>
#include <caja/libcaja-extension/caja-file-info.h>
#include <caja/libcaja-extension/caja-menu-provider.h>
#include "caja-x2goumount.h"

static GObjectClass *parent_class = NULL;
static GType x2goumount_type = 0;

GType caja_x2goumount_get_type (void) 
{
	return x2goumount_type;
}

#ifndef HAVE_CAJA_MENU_PROVIDER_EMIT_ITEMS_UPDATED_SIGNAL
static void caja_menu_provider_emit_items_updated_signal (CajaMenuProvider *provider)
{
}
#endif

static void caja_x2goumount_execute (CajaMenuItem *item)
{
	GList *files;
	files = (GList*)g_object_get_data (G_OBJECT (item), "files");
	gchar* uri = caja_file_info_get_uri ((CajaFileInfo *)files->data);
	gchar *file_path = g_filename_from_uri (uri, NULL, NULL);
	char* cmd=malloc(strlen(file_path)+strlen("x2goumount-session-mate"));
	strcpy(cmd,"x2goumount-session-mate ");
	strcat(cmd, file_path);
	g_spawn_command_line_async (cmd, NULL);
	g_free (uri);
	g_free (file_path);
}

static const gchar* get_verified_icon_name (const gchar* icon_name)
{
	if (icon_name[0] == '/')
	{
		if (!g_file_test (icon_name, G_FILE_TEST_IS_REGULAR))
		{
			return NULL;
		}
	}
	else if (strlen (icon_name) == 0)
	{
		return NULL;
	}

	return icon_name;
}

static CajaMenuItem *caja_x2goumount_create_menu_item (GList *files)
{
	CajaMenuItem *item;

	
	item = caja_menu_item_new ("CajaX2GoUmount::umount", 
	                          _("Unmount SSHFS Drive"), 
	                          _("Unmount SSHFS Drive"), 
	                          0l);

	g_signal_connect_data (item, 
	                       "activate",
	                       G_CALLBACK (caja_x2goumount_execute),
	                       0,
	                       0,
	                       0);

	g_object_set_data_full (G_OBJECT (item),
	        "files",
	        caja_file_info_list_copy (files),
	        (GDestroyNotify) caja_file_info_list_free);
	return item;
}

static GList *caja_x2goumount_get_file_items (CajaMenuProvider *provider, GtkWidget *window, GList *files)
{
	GList *items = NULL;
	int create_menu=FALSE;
	int f_items=0;	
	CajaMenuItem *item;
	CajaX2GoUmount* self = CAJA_X2GOUMOUNT (provider);

	g_return_val_if_fail (CAJA_IS_X2GOUMOUNT (self), NULL);


	if (!self->dispose_has_run)
	{
		GList* iter1;
		for (iter1 = files; iter1; iter1 = iter1->next)
		{
			f_items++;
			gchar* tmp_filename = caja_file_info_get_name ((CajaFileInfo *)iter1->data);
			if (tmp_filename)
			{
				gchar* tmp_mimetype = caja_file_info_get_mime_type ((CajaFileInfo *)iter1->data);
				if(strstr(tmp_mimetype,"sshfs"))
				{
					create_menu=TRUE;
				}
				g_free (tmp_filename);
				g_free (tmp_mimetype);
			}
		}
		if( f_items==1 && create_menu)
		{
			item = caja_x2goumount_create_menu_item (files);
			items = g_list_append (items, item);
		}
	}
	return items;
}

static GList *caja_x2goumount_get_background_items (CajaMenuProvider *provider, GtkWidget *window, CajaFileInfo *current_folder)
{
	GList *items = NULL;
	GList *files = NULL;

	files = g_list_append (files, current_folder);
	items = caja_x2goumount_get_file_items (provider, window, files);
	g_list_free (files);
	
	return items;
}

static void caja_x2goumount_instance_dispose (GObject *obj)
{
	CajaX2GoUmount* self = CAJA_X2GOUMOUNT (obj);
	
	if (!self->dispose_has_run)
	{
		self->dispose_has_run = TRUE;
		G_OBJECT_CLASS (parent_class)->dispose (obj);
	}
}


static void caja_x2goumount_instance_finalize (GObject* obj)
{
	G_OBJECT_CLASS (parent_class)->finalize (obj);
}

static void caja_x2goumount_class_init (CajaX2GoUmountClass *x2goumount_class)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (x2goumount_class);
	gobject_class->dispose = caja_x2goumount_instance_dispose;
	gobject_class->finalize = caja_x2goumount_instance_finalize;
}

static void caja_x2goumount_instance_init (GTypeInstance *instance, gpointer klass)
{
	CajaX2GoUmount* self = CAJA_X2GOUMOUNT (instance);
	self->dispose_has_run = FALSE;
	parent_class = g_type_class_peek_parent (klass);
}

static void caja_x2goumount_menu_provider_iface_init (CajaMenuProviderIface *iface)
{
	iface->get_file_items = caja_x2goumount_get_file_items;
	iface->get_background_items = caja_x2goumount_get_background_items;
}

void caja_x2goumount_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
	        sizeof (CajaX2GoUmountClass),
	        (GBaseInitFunc) NULL,
	        (GBaseFinalizeFunc) NULL,
	        (GClassInitFunc) caja_x2goumount_class_init,
	        NULL,
	        NULL,
	        sizeof (CajaX2GoUmount),
	        0,
	        (GInstanceInitFunc)caja_x2goumount_instance_init,
	};

	static const GInterfaceInfo menu_provider_iface_info = {
	        (GInterfaceInitFunc) caja_x2goumount_menu_provider_iface_init,
	        NULL,
	        NULL
	};

	x2goumount_type = g_type_module_register_type (module,
	                                               G_TYPE_OBJECT,
	                                               "CajaX2GoUmount",
	                                               &info, 0);

	g_type_module_add_interface (module,
	                             x2goumount_type,
	                             CAJA_TYPE_MENU_PROVIDER,
	                             &menu_provider_iface_info);
}

